/*
 Name:		AlexaServer
 Created:	26.01.2024
 Author:	bieno
*/

#include <WiFi.h>
#include <WebServer.h>
#include <UrsWiFi.h>
#include <UrsEsp.h>
#include <UrsOTA.h>
#include "AppVersion.h"

using namespace UrsCPU;

const char* ssid = "MyWiFi"; 
const char* password = "MyPassword";
const char* hostname = "AlexaServer";
const IPAddress ip(192, ---, ---, --);
const int   port = 7235;

WebServer server(port);

void setup() {
   // Serielle Schnittstelle initialisieren
   // -------------------------------------------------------------------------
   Serial.begin(115200);

   Serial.println("\n" APP_NAME " gestartet. Version: " APP_VERSION "\n");

   while (Serial.available()) // Falls es Schmutz auf der Leitung gibt.
      Serial.read();

   // Reset-Grund abprfen
   // -------------------------------------------------------------------------
   int resetReason = UrsESP.getResetReason(0);
   Serial.printf("CPU0 Reset Grund: %i (%s) %s\n", resetReason, UrsESP.getResetReasonName(resetReason).c_str(), UrsESP.getResetReasonDescription(resetReason).c_str());

   // Interne Fehler fhren dauernd zu einem neuen Reset
   // Reason 1:  POWER_ON_RESET
   // Reason 12: SW_CPU_RESET (z.B. nach OTA)
   if (resetReason != 1 && resetReason != 12) 
      while (1);

   // mit WLAN verbinden
   // -------------------------------------------------------------------------
   auto conStatus = UrsWiFi.connectStation(ssid, password, hostname, ip);
   if (!(conStatus & UrsCodes::ConnectCode::notConnected)) { // Verbunden
      if (conStatus != UrsCodes::ConnectCode::noError) {// aber Probleme: 2. Versuch
         Serial.println("=== 2. Versuch");
         UrsWiFi.connectStation(ssid, password, hostname, ip);
      }
   }

   // Web-Server konfigurieren
   // -------------------------------------------------------------------------
   server.on("/tv", handleTvRequest);
   server.on("/channel", handleChannelRequest);
   server.onNotFound(handleNotFound);

   // Server starten
   // -------------------------------------------------------------------------
   server.begin();
   Serial.printf("HTTP Server gestartet auf Port %i\n", port);

   // OTA starten
   // -------------------------------------------------------------------------
   UrsOTA.begin(hostname);
}


void loop() {
   server.handleClient();
   UrsOTA.handle();
}

// Handler fr HTTP-Requests
// -------------------------------------------------------------------------
void handleRootRequest() {
   Serial.printf("RootRequest from %s, Argumente sind %s %s %s\n", server.client().remoteIP().toString().c_str(),
      server.arg(0).c_str(), server.arg(1).c_str(), server.arg(2).c_str());
   server.send(200, "text/html", "Fernseher " + server.arg(0) + " geschaltet.");
}

void handleTvRequest() {
   Serial.printf("TvRequest from %s, Argument ist %s\n", server.client().remoteIP().toString().c_str(), server.arg(0).c_str());
   server.send(200, "text/html", "Fernseher " + server.arg(0) + " geschaltet.");
}

void handleChannelRequest() {
   Serial.printf("ChannelRequest from %s, Argument ist %s\n", server.client().remoteIP().toString().c_str(), server.arg(0).c_str());
   server.send(200, "text/html", "Programm " + server.arg(0) + " eingeschaltet.");
}

void handleNotFound() {
   server.send(404, "text/plain", "ich verstehe nicht, was gemeint ist.");
}
